@@ -1,17 +1,12 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
 #ifndef DNS_VIEW_H
@@ -58,6 +53,8 @@
  */
 
 #include <stdio.h>
+#include <inttypes.h>
+#include <stdbool.h>
 
 #include <isc/lang.h>
 #include <isc/magic.h>
@@ -69,6 +66,9 @@
 #include <isc/stdtime.h>
 
 #include <dns/acl.h>
+#include <dns/catz.h>
+#include <dns/clientinfo.h>
+#include <dns/dnstap.h>
 #include <dns/fixedname.h>
 #include <dns/rrl.h>
 #include <dns/rdatastruct.h>
@@ -85,7 +85,6 @@ struct dns_view {
 	dns_rdataclass_t		rdclass;
 	char *				name;
 	dns_zt_t *			zonetable;
-	dns_dlzdb_t *			dlzdatabase;
 	dns_resolver_t *		resolver;
 	dns_adb_t *			adb;
 	dns_requestmgr_t *		requestmgr;
@@ -95,20 +94,22 @@ struct dns_view {
 	dns_db_t *			hints;
 
 	/*
-	 * security roots.
+	 * security roots and negative trust anchors.
 	 * internal use only; access via * dns_view_getsecroots()
 	 */
 	dns_keytable_t *		secroots_priv;
+	dns_ntatable_t *		ntatable_priv;
 
 	isc_mutex_t			lock;
-	isc_boolean_t			frozen;
+	bool			frozen;
 	isc_task_t *			task;
 	isc_event_t			resevent;
 	isc_event_t			adbevent;
 	isc_event_t			reqevent;
+	isc_stats_t *			adbstats;
 	isc_stats_t *			resstats;
 	dns_stats_t *			resquerystats;
-	isc_boolean_t			cacheshared;
+	bool			cacheshared;
 
 	/* Configurable data. */
 	dns_tsig_keyring_t *		statickeys;
@@ -116,15 +117,18 @@ struct dns_view {
 	dns_peerlist_t *		peers;
 	dns_order_t *			order;
 	dns_fwdtable_t *		fwdtable;
-	isc_boolean_t			recursion;
-	isc_boolean_t			auth_nxdomain;
-	isc_boolean_t			additionalfromcache;
-	isc_boolean_t			additionalfromauth;
-	isc_boolean_t			minimalresponses;
-	isc_boolean_t			enablednssec;
-	isc_boolean_t			enablevalidation;
-	isc_boolean_t			acceptexpired;
-	isc_boolean_t			root_key_sentinel;
+	bool			recursion;
+	bool			auth_nxdomain;
+	bool			additionalfromcache;
+	bool			additionalfromauth;
+	bool			minimal_any;
+	dns_minimaltype_t		minimalresponses;
+	bool			enablednssec;
+	bool			enablevalidation;
+	bool			acceptexpired;
+	bool			requireservercookie;
+	bool			trust_anchor_telemetry;
+	bool			root_key_sentinel;
 	dns_transfer_format_t		transfer_format;
 	dns_acl_t *			cacheacl;
 	dns_acl_t *			cacheonacl;
@@ -139,34 +143,45 @@ struct dns_view {
 	dns_acl_t *			upfwdacl;
 	dns_acl_t *			denyansweracl;
 	dns_acl_t *			nocasecompress;
+	bool			msgcompression;
 	dns_rbt_t *			answeracl_exclude;
 	dns_rbt_t *			denyanswernames;
 	dns_rbt_t *			answernames_exclude;
 	dns_rrl_t *			rrl;
-	isc_boolean_t			provideixfr;
-	isc_boolean_t			requestnsid;
+	bool			provideixfr;
+	bool			requestnsid;
+	bool			sendcookie;
 	dns_ttl_t			maxcachettl;
 	dns_ttl_t			maxncachettl;
+	uint32_t			nta_lifetime;
+	uint32_t			nta_recheck;
+	char				*nta_file;
+	dns_ttl_t			prefetch_trigger;
+	dns_ttl_t			prefetch_eligible;
 	in_port_t			dstport;
 	dns_aclenv_t			aclenv;
 	dns_rdatatype_t			preferred_glue;
-	isc_boolean_t			flush;
+	bool			flush;
 	dns_namelist_t *		delonly;
-	isc_boolean_t			rootdelonly;
+	bool			rootdelonly;
 	dns_namelist_t *		rootexclude;
-	isc_boolean_t			checknames;
+	bool			checknames;
 	dns_name_t *			dlv;
 	dns_fixedname_t			dlv_fixed;
-	isc_uint16_t			maxudp;
+	uint16_t			maxudp;
+	uint16_t			nocookieudp;
 	unsigned int			maxbits;
-	dns_v4_aaaa_t			v4_aaaa;
-	dns_acl_t *			v4_aaaa_acl;
+	dns_aaaa_t			v4_aaaa;
+	dns_aaaa_t			v6_aaaa;
+	dns_acl_t *			aaaa_acl;
 	dns_dns64list_t 		dns64;
 	unsigned int 			dns64cnt;
-	ISC_LIST(dns_rpz_zone_t)	rpz_zones;
-	isc_boolean_t			rpz_recursive_only;
-	isc_boolean_t			rpz_break_dnssec;
-	unsigned int			rpz_min_ns_labels;
+	dns_rpz_zones_t			*rpzs;
+	dns_catz_zones_t		*catzs;
+	dns_dlzdblist_t 		dlz_searched;
+	dns_dlzdblist_t 		dlz_unsearched;
+	uint32_t			fail_ttl;
+	dns_badcache_t			*failcache;
 
 	/*
 	 * Configurable data for server use only,
@@ -174,7 +189,7 @@ struct dns_view {
 	 */
 	dns_acl_t *			matchclients;
 	dns_acl_t *			matchdestinations;
-	isc_boolean_t			matchrecursiveonly;
+	bool			matchrecursiveonly;
 
 	/* Locked by themselves. */
 	isc_refcount_t			references;
@@ -188,14 +203,32 @@ struct dns_view {
 
 	dns_zone_t *			managed_keys;
 	dns_zone_t *			redirect;
+	dns_name_t *			redirectzone;	/* points to
+							 * redirectfixed
+							 * when valid */
+	dns_fixedname_t 		redirectfixed;
 
-#ifdef BIND9
-	/* File in which to store configuration for newly added zones */
+	/*
+	 * File and configuration data for zones added at runtime
+	 * (only used in BIND9).
+	 *
+	 * XXX: This should be a pointer to an opaque type that
+	 * named implements.
+	 */
 	char *				new_zone_file;
-
+	char *			        new_zone_db;
+	void *				new_zone_dbenv;
+	uint64_t			new_zone_mapsize;
 	void *				new_zone_config;
 	void				(*cfg_destroy)(void **);
-#endif
+	isc_mutex_t			new_zone_lock;
+
+	unsigned char			secret[32];	/* Client secret */
+	unsigned int			v6bias;
+
+	dns_dtenv_t			*dtenv;		/* Dnstap environment */
+	dns_dtmsgtype_t			dttypes;	/* Dnstap message types
+							   to log */
 };
 
 #define DNS_VIEW_MAGIC			ISC_MAGIC('V','i','e','w')
@@ -205,6 +238,25 @@ struct dns_view {
 #define DNS_VIEWATTR_ADBSHUTDOWN	0x02
 #define DNS_VIEWATTR_REQSHUTDOWN	0x04
 
+#ifdef HAVE_LMDB
+#include <lmdb.h>
+/*
+ * MDB_NOTLS is used to prevent problems after configuration is reloaded, due
+ * to the way LMDB's use of thread-local storage (TLS) interacts with the BIND9
+ * thread model.
+ */
+#define DNS_LMDB_COMMON_FLAGS		(MDB_CREATE | MDB_NOSUBDIR | MDB_NOTLS)
+#ifndef __OpenBSD__
+#define DNS_LMDB_FLAGS			(DNS_LMDB_COMMON_FLAGS)
+#else /* __OpenBSD__ */
+/*
+ * OpenBSD does not have a unified buffer cache, which requires both reads and
+ * writes to be performed using mmap().
+ */
+#define DNS_LMDB_FLAGS			(DNS_LMDB_COMMON_FLAGS | MDB_WRITEMAP)
+#endif /* __OpenBSD__ */
+#endif /* HAVE_LMDB */
+
 isc_result_t
 dns_view_create(isc_mem_t *mctx, dns_rdataclass_t rdclass,
 		const char *name, dns_view_t **viewp);
@@ -313,6 +365,24 @@ dns_view_weakdetach(dns_view_t **targetp);
  *\li	*viewp is NULL.
  */
 
+isc_result_t
+dns_view_createzonetable(dns_view_t *view);
+/*%<
+ * Create a zonetable for the view.
+ *
+ * Requires:
+ *
+ *\li	'view' is a valid, unfrozen view.
+ *
+ *\li	'view' does not have a zonetable already.
+ *
+ * Returns:
+ *
+ *\li   	#ISC_R_SUCCESS
+ *
+ *\li	Any error that dns_zt_create() can return.
+ */
+
 isc_result_t
 dns_view_createresolver(dns_view_t *view,
 			isc_taskmgr_t *taskmgr,
@@ -346,11 +416,11 @@ dns_view_createresolver(dns_view_t *view,
 void
 dns_view_setcache(dns_view_t *view, dns_cache_t *cache);
 void
-dns_view_setcache2(dns_view_t *view, dns_cache_t *cache, isc_boolean_t shared);
+dns_view_setcache2(dns_view_t *view, dns_cache_t *cache, bool shared);
 /*%<
  * Set the view's cache database.  If 'shared' is true, this means the cache
  * is created by another view and is shared with that view.  dns_view_setcache()
- * is a backward compatible version equivalent to setcache2(..., ISC_FALSE).
+ * is a backward compatible version equivalent to setcache2(..., false).
  *
  * Requires:
  *
@@ -476,13 +546,13 @@ dns_view_thaw(dns_view_t *view);
  */
 isc_result_t
 dns_view_find(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
-	      isc_stdtime_t now, unsigned int options, isc_boolean_t use_hints,
+	      isc_stdtime_t now, unsigned int options, bool use_hints,
 	      dns_db_t **dbp, dns_dbnode_t **nodep, dns_name_t *foundname,
 	      dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset);
 isc_result_t
 dns_view_find2(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
 	       isc_stdtime_t now, unsigned int options,
-	       isc_boolean_t use_hints, isc_boolean_t use_static_stub,
+	       bool use_hints, bool use_static_stub,
 	       dns_db_t **dbp, dns_dbnode_t **nodep, dns_name_t *foundname,
 	       dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset);
 /*%<
@@ -490,13 +560,13 @@ dns_view_find2(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
  * 'type'.
  * In general, this function first searches view's zone and cache DBs for the
  * best match data against 'name'.  If nothing found there, and if 'use_hints'
- * is ISC_TRUE, the view's hint DB (if configured) is searched.
+ * is true, the view's hint DB (if configured) is searched.
  * If the view is configured with a static-stub zone which gives the longest
  * match for 'name' among the zones, however, the cache DB is not consulted
- * unless 'use_static_stub' is ISC_FALSE (see below about this argument).
+ * unless 'use_static_stub' is false (see below about this argument).
  *
  * dns_view_find() is a backward compatible version equivalent to
- * dns_view_find2() with use_static_stub argument being ISC_FALSE.
+ * dns_view_find2() with use_static_stub argument being false.
  *
  * Notes:
  *
@@ -506,16 +576,16 @@ dns_view_find2(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
  *
  *\li	If 'now' is zero, then the current time will be used.
  *
- *\li	If 'use_hints' is ISC_TRUE, and the view has a hints database, then
+ *\li	If 'use_hints' is true, and the view has a hints database, then
  *	it will be searched last.  If the answer is found in the hints
  *	database, the result code will be DNS_R_HINT.  If the name is found
  *	in the hints database but not the type, the result code will be
  *	#DNS_R_HINTNXRRSET.
  *
- *\li	If 'use_static_stub' is ISC_FALSE and the longest match zone for 'name'
+ *\li	If 'use_static_stub' is false and the longest match zone for 'name'
  *	is a static-stub zone, it's ignored and the cache and/or hints will be
  *	searched.  In the majority of the cases this argument should be
- *	ISC_FALSE.  The only known usage of this argument being ISC_TRUE is
+ *	false.  The only known usage of this argument being true is
  *	if this search is for a "bailiwick" glue A or AAAA RRset that may
  *	best match a static-stub zone.  Consider the following example:
  *	this view is configured with a static-stub zone "example.com",
@@ -573,7 +643,7 @@ dns_view_find2(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
 isc_result_t
 dns_view_simplefind(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
 		    isc_stdtime_t now, unsigned int options,
-		    isc_boolean_t use_hints,
+		    bool use_hints,
 		    dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset);
 /*%<
  * Find an rdataset whose owner name is 'name', and whose type is
@@ -591,7 +661,7 @@ dns_view_simplefind(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
  *
  *\li	If 'now' is zero, then the current time will be used.
  *
- *\li	If 'use_hints' is ISC_TRUE, and the view has a hints database, then
+ *\li	If 'use_hints' is true, and the view has a hints database, then
  *	it will be searched last.  If the answer is found in the hints
  *	database, the result code will be DNS_R_HINT.  If the name is found
  *	in the hints database but not the type, the result code will be
@@ -635,13 +705,13 @@ dns_view_simplefind(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
 isc_result_t
 dns_view_findzonecut(dns_view_t *view, dns_name_t *name, dns_name_t *fname,
 		     isc_stdtime_t now, unsigned int options,
-		     isc_boolean_t use_hints,
+		     bool use_hints,
 		     dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset);
 
 isc_result_t
 dns_view_findzonecut2(dns_view_t *view, dns_name_t *name, dns_name_t *fname,
 		      isc_stdtime_t now, unsigned int options,
-		      isc_boolean_t use_hints, isc_boolean_t use_cache,
+		      bool use_hints, bool use_cache,
 		      dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset);
 /*%<
  * Find the best known zonecut containing 'name'.
@@ -653,10 +723,10 @@ dns_view_findzonecut2(dns_view_t *view, dns_name_t *name, dns_name_t *fname,
  *
  *\li	If 'now' is zero, then the current time will be used.
  *
- *\li	If 'use_hints' is ISC_TRUE, and the view has a hints database, then
+ *\li	If 'use_hints' is true, and the view has a hints database, then
  *	it will be searched last.
  *
- *\li	If 'use_cache' is ISC_TRUE, and the view has a cache, then it will be
+ *\li	If 'use_cache' is true, and the view has a cache, then it will be
  *	searched.
  *
  *\li	If 'sigrdataset' is not NULL, and there is a SIG rdataset which
@@ -700,7 +770,7 @@ dns_viewlist_find(dns_viewlist_t *list, const char *name,
  */
 
 isc_result_t
-dns_viewlist_findzone(dns_viewlist_t *list, dns_name_t *name, isc_boolean_t allclasses,
+dns_viewlist_findzone(dns_viewlist_t *list, dns_name_t *name, bool allclasses,
 		      dns_rdataclass_t rdclass, dns_zone_t **zonep);
 
 /*%<
@@ -731,13 +801,17 @@ dns_view_findzone(dns_view_t *view, dns_name_t *name, dns_zone_t **zonep);
  */
 
 isc_result_t
-dns_view_load(dns_view_t *view, isc_boolean_t stop);
+dns_view_load(dns_view_t *view, bool stop);
 
 isc_result_t
-dns_view_loadnew(dns_view_t *view, isc_boolean_t stop);
+dns_view_loadnew(dns_view_t *view, bool stop);
 
 isc_result_t
 dns_view_asyncload(dns_view_t *view, dns_zt_allloaded_t callback, void *arg);
+
+isc_result_t
+dns_view_asyncload2(dns_view_t *view, dns_zt_allloaded_t callback, void *arg,
+		    bool newonly);
 /*%<
  * Load zones attached to this view.  dns_view_load() loads
  * all zones whose master file has changed since the last
@@ -748,8 +822,8 @@ dns_view_asyncload(dns_view_t *view, dns_zt_allloaded_t callback, void *arg);
  * in the view have finished loading, 'callback' is called with argument
  * 'arg' to inform the caller.
  *
- * If 'stop' is ISC_TRUE, stop on the first error and return it.
- * If 'stop' is ISC_FALSE (or we are loading asynchronously), ignore errors.
+ * If 'stop' is true, stop on the first error and return it.
+ * If 'stop' is false (or we are loading asynchronously), ignore errors.
  *
  * Requires:
  *
@@ -834,7 +908,7 @@ dns_view_dumpdbtostream(dns_view_t *view, FILE *fp);
 isc_result_t
 dns_view_flushcache(dns_view_t *view);
 isc_result_t
-dns_view_flushcache2(dns_view_t *view, isc_boolean_t fixuponly);
+dns_view_flushcache2(dns_view_t *view, bool fixuponly);
 /*%<
  * Flush the view's cache (and ADB).  If 'fixuponly' is true, it only updates
  * the internal reference to the cache DB with omitting actual flush operation.
@@ -853,15 +927,12 @@ dns_view_flushcache2(dns_view_t *view, isc_boolean_t fixuponly);
  */
 
 isc_result_t
-dns_view_flushnode(dns_view_t *view, dns_name_t *name, isc_boolean_t tree);
+dns_view_flushnode(dns_view_t *view, dns_name_t *name, bool tree);
 /*%<
  * Flush the given name from the view's cache (and optionally ADB/badcache).
  *
- * If 'tree' is true, flush 'name' and all names below it
- * from the cache, but do not flush ADB.
- *
- * If 'tree' is false, flush 'name' frmo both the cache and ADB,
- * but do not touch any other nodes.
+ * Flush the given name from the cache, ADB, and bad cache.  If 'tree'
+ * is true, also flush all subdomains of 'name'.
  *
  * Requires:
  *\li	'view' is valid.
@@ -876,7 +947,7 @@ isc_result_t
 dns_view_flushname(dns_view_t *view, dns_name_t *name);
 /*%<
  * Flush the given name from the view's cache, ADB and badcache.
- * Equivalent to dns_view_flushnode(view, name, ISC_FALSE).
+ * Equivalent to dns_view_flushnode(view, name, false).
  *
  *
  * Requires:
@@ -917,7 +988,7 @@ dns_view_excludedelegationonly(dns_view_t *view, dns_name_t *name);
  *\li	#ISC_R_NOMEMORY
  */
 
-isc_boolean_t
+bool
 dns_view_isdelegationonly(dns_view_t *view, dns_name_t *name);
 /*%<
  * Check if 'name' is in the delegation only table or if
@@ -928,12 +999,12 @@ dns_view_isdelegationonly(dns_view_t *view, dns_name_t *name);
  *\li	'name' is valid.
  *
  * Returns:
- *\li	#ISC_TRUE if the name is the table.
- *\li	#ISC_FALSE otherwise.
+ *\li	#true if the name is the table.
+ *\li	#false otherwise.
  */
 
 void
-dns_view_setrootdelonly(dns_view_t *view, isc_boolean_t value);
+dns_view_setrootdelonly(dns_view_t *view, bool value);
 /*%<
  * Set the root delegation only flag.
  *
@@ -941,7 +1012,7 @@ dns_view_setrootdelonly(dns_view_t *view, isc_boolean_t value);
  *\li	'view' is valid.
  */
 
-isc_boolean_t
+bool
 dns_view_getrootdelonly(dns_view_t *view);
 /*%<
  * Get the root delegation only flag.
@@ -951,7 +1022,7 @@ dns_view_getrootdelonly(dns_view_t *view);
  */
 
 isc_result_t
-dns_view_freezezones(dns_view_t *view, isc_boolean_t freeze);
+dns_view_freezezones(dns_view_t *view, bool freeze);
 /*%<
  * Freeze/thaw updates to master zones.
  *
@@ -959,6 +1030,31 @@ dns_view_freezezones(dns_view_t *view, isc_boolean_t freeze);
  * \li	'view' is valid.
  */
 
+void
+dns_view_setadbstats(dns_view_t *view, isc_stats_t *stats);
+/*%<
+ * Set a adb statistics set 'stats' for 'view'.
+ *
+ * Requires:
+ * \li	'view' is valid and is not frozen.
+ *
+ *\li	stats is a valid statistics supporting adb statistics
+ *	(see dns/stats.h).
+ */
+
+void
+dns_view_getadbstats(dns_view_t *view, isc_stats_t **statsp);
+/*%<
+ * Get the adb statistics counter set for 'view'.  If a statistics set is
+ * set '*statsp' will be attached to the set; otherwise, '*statsp' will be
+ * untouched.
+ *
+ * Requires:
+ * \li	'view' is valid and is not frozen.
+ *
+ *\li	'statsp' != NULL && '*statsp' != NULL
+ */
+
 void
 dns_view_setresstats(dns_view_t *view, isc_stats_t *stats);
 /*%<
@@ -1010,7 +1106,7 @@ dns_view_getresquerystats(dns_view_t *view, dns_stats_t **statsp);
  *\li	'statsp' != NULL && '*statsp' != NULL
  */
 
-isc_boolean_t
+bool
 dns_view_iscacheshared(dns_view_t *view);
 /*%<
  * Check if the view shares the cache created by another view.
@@ -1019,21 +1115,54 @@ dns_view_iscacheshared(dns_view_t *view);
  * \li	'view' is valid.
  *
  * Returns:
- *\li	#ISC_TRUE if the cache is shared.
- *\li	#ISC_FALSE otherwise.
+ *\li	#true if the cache is shared.
+ *\li	#false otherwise.
+ */
+
+isc_result_t
+dns_view_initntatable(dns_view_t *view,
+		      isc_taskmgr_t *taskmgr, isc_timermgr_t *timermgr);
+/*%<
+ * Initialize the negative trust anchor table for the view.
+ *
+ * Requires:
+ * \li	'view' is valid.
+ *
+ * Returns:
+ *\li	ISC_R_SUCCESS
+ *\li	Any other result indicates failure
+ */
+
+isc_result_t
+dns_view_getntatable(dns_view_t *view, dns_ntatable_t **ntp);
+/*%<
+ * Get the negative trust anchor table for this view.  Returns
+ * ISC_R_NOTFOUND if the table not been initialized for the view.
+ *
+ * '*ntp' is attached on success; the caller is responsible for
+ * detaching it with dns_ntatable_detach().
+ *
+ * Requires:
+ * \li	'view' is valid.
+ * \li	'nta' is not NULL and '*nta' is NULL.
+ *
+ * Returns:
+ *\li	ISC_R_SUCCESS
+ *\li	ISC_R_NOTFOUND
  */
 
 isc_result_t
 dns_view_initsecroots(dns_view_t *view, isc_mem_t *mctx);
 /*%<
- * Initialize security roots for the view.  (Note that secroots is
+ * Initialize security roots for the view, detaching any previously
+ * existing security roots first.  (Note that secroots_priv is
  * NULL until this function is called, so any function using
- * secroots must check its validity first.  One way to do this is
- * use dns_view_getsecroots() and check its return value.)
+ * security roots must check that they have been initialized first.
+ * One way to do this is use dns_view_getsecroots() and check its
+ * return value.)
  *
  * Requires:
  * \li	'view' is valid.
- * \li	'view->secroots' is NULL.
  *
  * Returns:
  *\li	ISC_R_SUCCESS
@@ -1060,10 +1189,14 @@ dns_view_getsecroots(dns_view_t *view, dns_keytable_t **ktp);
 
 isc_result_t
 dns_view_issecuredomain(dns_view_t *view, dns_name_t *name,
-			 isc_boolean_t *secure_domain);
+			isc_stdtime_t now, bool checknta,
+			bool *secure_domain);
 /*%<
- * Is 'name' at or beneath a trusted key?  Put answer in
- * '*secure_domain'.
+ * Is 'name' at or beneath a trusted key, and not covered by a valid
+ * negative trust anchor?  Put answer in '*secure_domain'.
+ *
+ * If 'checknta' is false, ignore the NTA table in determining
+ * whether this is a secure domain.
  *
  * Requires:
  * \li	'view' is valid.
@@ -1073,6 +1206,20 @@ dns_view_issecuredomain(dns_view_t *view, dns_name_t *name,
  *\li	Any other value indicates failure
  */
 
+bool
+dns_view_ntacovers(dns_view_t *view, isc_stdtime_t now,
+		   dns_name_t *name, dns_name_t *anchor);
+/*%<
+ * Is there a current negative trust anchor above 'name' and below 'anchor'?
+ *
+ * Requires:
+ * \li	'view' is valid.
+ *
+ * Returns:
+ *\li	ISC_R_TRUE
+ *\li	ISC_R_FALSE
+ */
+
 void
 dns_view_untrust(dns_view_t *view, dns_name_t *keyname,
 		 dns_rdata_dnskey_t *dnskey, isc_mem_t *mctx);
@@ -1094,12 +1241,12 @@ dns_view_untrust(dns_view_t *view, dns_name_t *keyname,
  */
 
 isc_result_t
-dns_view_setnewzones(dns_view_t *view, isc_boolean_t allow, void *cfgctx,
-		     void (*cfg_destroy)(void **));
+dns_view_setnewzones(dns_view_t *view, bool allow, void *cfgctx,
+		     void (*cfg_destroy)(void **), uint64_t mapsize);
 /*%<
  * Set whether or not to allow zones to be created or deleted at runtime.
  *
- * If 'allow' is ISC_TRUE, determines the filename into which new zone
+ * If 'allow' is true, determines the filename into which new zone
  * configuration will be written.  Preserves the configuration context
  * (a pointer to which is passed in 'cfgctx') for use when parsing new
  * zone configuration.  'cfg_destroy' points to a callback routine to
@@ -1107,7 +1254,7 @@ dns_view_setnewzones(dns_view_t *view, isc_boolean_t allow, void *cfgctx,
  * roundabout method is used in order to avoid libdns having a dependency
  * on libisccfg and libbind9.)
  *
- * If 'allow' is ISC_FALSE, removes any existing references to
+ * If 'allow' is false, removes any existing references to
  * configuration context and frees any memory.
  *
  * Requires:
@@ -1115,12 +1262,71 @@ dns_view_setnewzones(dns_view_t *view, isc_boolean_t allow, void *cfgctx,
  *
  * Returns:
  * \li ISC_R_SUCCESS
- * \li ISC_R_NOMEMORY
+ * \li ISC_R_NOSPACE
  */
 
 void
 dns_view_restorekeyring(dns_view_t *view);
 
+isc_result_t
+dns_view_searchdlz(dns_view_t *view, dns_name_t *name,
+		   unsigned int minlabels,
+		   dns_clientinfomethods_t *methods,
+		   dns_clientinfo_t *clientinfo,
+		   dns_db_t **dbp);
+
+/*%<
+ * Search through the DLZ database(s) in view->dlz_searched to find
+ * one that can answer a query for 'name', using the DLZ driver's
+ * findzone method.  If successful, '*dbp' is set to point to the
+ * DLZ database.
+ *
+ * Returns:
+ * \li ISC_R_SUCCESS
+ * \li ISC_R_NOTFOUND
+ *
+ * Requires:
+ * \li 'view' is valid.
+ * \li 'name' is not NULL.
+ * \li 'dbp' is not NULL and *dbp is NULL.
+ */
+
+uint32_t
+dns_view_getfailttl(dns_view_t *view);
+/*%<
+ * Get the view's servfail-ttl.  zero => no servfail caching.
+ *
+ * Requires:
+ *\li	'view' to be valid.
+ */
+
+void
+dns_view_setfailttl(dns_view_t *view, uint32_t failttl);
+/*%<
+ * Set the view's servfail-ttl.  zero => no servfail caching.
+ *
+ * Requires:
+ *\li	'view' to be valid.
+ */
+
+isc_result_t
+dns_view_saventa(dns_view_t *view);
+/*%<
+ * Save NTA for names in this view to a file.
+ *
+ * Requires:
+ *\li	'view' to be valid.
+ */
+
+isc_result_t
+dns_view_loadnta(dns_view_t *view);
+/*%<
+ * Loads NTA for names in this view from a file.
+ *
+ * Requires:
+ *\li	'view' to be valid.
+ */
+
 void
 dns_view_setviewcommit(dns_view_t *view);
 /*%<
@@ -1141,6 +1347,7 @@ dns_view_setviewrevert(dns_view_t *view);
  *\li	'view' to be valid.
  */
 
+
 ISC_LANG_ENDDECLS
 
 #endif /* DNS_VIEW_H */
