@@ -1,14 +1,19 @@
 /*
- * Copyright (C) 2014-2017  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ *
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
 #include <config.h>
 
 #include <stdio.h>
+#include <inttypes.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -18,6 +23,7 @@
 #include <isc/platform.h>
 #include <isc/print.h>
 #include <isc/stdio.h>
+#include <isc/string.h>
 #include <isc/thread.h>
 #include <isc/util.h>
 
@@ -30,6 +36,7 @@
 
 #include <pkcs11/cryptoki.h>
 #include <pkcs11/pkcs11.h>
+#include <pkcs11/eddsa.h>
 
 /* was 32 octets, Petr Spacek suggested 1024, SoftHSMv2 uses 256... */
 #ifndef PINLEN
@@ -40,12 +47,12 @@
 #define PK11_NO_LOGERR 1
 #endif
 
-LIBISC_EXTERNAL_DATA isc_boolean_t pk11_verbose_init = ISC_FALSE;
+LIBISC_EXTERNAL_DATA bool pk11_verbose_init = false;
 
 static isc_once_t once = ISC_ONCE_INIT;
 static isc_mem_t *pk11_mctx = NULL;
-static isc_int32_t allocsize = 0;
-static isc_boolean_t initialized = ISC_FALSE;
+static int32_t allocsize = 0;
+static bool initialized = false;
 
 typedef struct pk11_session pk11_session_t;
 typedef struct pk11_token pk11_token_t;
@@ -64,7 +71,7 @@ struct pk11_token {
 	ISC_LINK(pk11_token_t)	link;
 	CK_SLOT_ID		slotid;
 	pk11_sessionlist_t	sessions;
-	isc_boolean_t		logged;
+	bool		logged;
 	char			name[32];
 	char			manuf[32];
 	char			model[16];
@@ -86,11 +93,11 @@ static isc_result_t free_all_sessions(void);
 static isc_result_t free_session_list(pk11_sessionlist_t *slist);
 static isc_result_t setup_session(pk11_session_t *sp,
 				  pk11_token_t *token,
-				  isc_boolean_t rw);
+				  bool rw);
 static void scan_slots(void);
 static isc_result_t token_login(pk11_session_t *sp);
 static char *percent_decode(char *x, size_t *len);
-static isc_boolean_t pk11strcmp(const char *x, size_t lenx,
+static bool pk11strcmp(const char *x, size_t lenx,
 				const char *y, size_t leny);
 static CK_ATTRIBUTE *push_attribute(pk11_object_t *obj,
 				    isc_mem_t *mctx,
@@ -178,21 +185,20 @@ pk11_mem_put(void *ptr, size_t size) {
 
 isc_result_t
 pk11_initialize(isc_mem_t *mctx, const char *engine) {
-	isc_result_t result;
+	isc_result_t result = ISC_R_SUCCESS;
 	CK_RV rv;
 
 	RUNTIME_CHECK(isc_once_do(&once, initialize) == ISC_R_SUCCESS);
 
+	LOCK(&sessionlock);
 	LOCK(&alloclock);
 	if ((mctx != NULL) && (pk11_mctx == NULL) && (allocsize == 0))
 		isc_mem_attach(mctx, &pk11_mctx);
+	UNLOCK(&alloclock);
 	if (initialized) {
-		UNLOCK(&alloclock);
-		return (ISC_R_SUCCESS);
+		goto unlock;
 	} else {
-		LOCK(&sessionlock);
-		initialized = ISC_TRUE;
-		UNLOCK(&alloclock);
+		initialized = true;
 	}
 
 	ISC_LIST_INIT(tokens);
@@ -232,7 +238,6 @@ pk11_initialize(isc_mem_t *mctx, const char *engine) {
 	}
 #endif
 #endif /* PKCS11CRYPTO */
-	result = ISC_R_SUCCESS;
  unlock:
 	UNLOCK(&sessionlock);
 	return (result);
@@ -270,7 +275,7 @@ pk11_finalize(void) {
 	}
 	if (pk11_mctx != NULL)
 		isc_mem_detach(&pk11_mctx);
-	initialized = ISC_FALSE;
+	initialized = false;
 	return (ret);
 }
 
@@ -280,8 +285,8 @@ pk11_rand_bytes(unsigned char *buf, int num) {
 	CK_RV rv;
 	pk11_context_t ctx;
 
-	ret = pk11_get_session(&ctx, OP_RAND, ISC_FALSE, ISC_FALSE,
-			       ISC_FALSE, NULL, 0);
+	ret = pk11_get_session(&ctx, OP_RAND, false, false,
+			       false, NULL, 0);
 	if ((ret != ISC_R_SUCCESS) &&
 	    (ret != PK11_R_NODIGESTSERVICE) &&
 	    (ret != PK11_R_NOAESSERVICE))
@@ -307,8 +312,8 @@ pk11_rand_seed_fromfile(const char *randomfile) {
 	size_t cc = 0;
 	isc_result_t ret;
 
-	ret = pk11_get_session(&ctx, OP_RAND, ISC_FALSE, ISC_FALSE,
-			       ISC_FALSE, NULL, 0);
+	ret = pk11_get_session(&ctx, OP_RAND, false, false,
+			       false, NULL, 0);
 	if ((ret != ISC_R_SUCCESS) &&
 	    (ret != PK11_R_NODIGESTSERVICE) &&
 	    (ret != PK11_R_NOAESSERVICE))
@@ -334,8 +339,8 @@ pk11_rand_seed_fromfile(const char *randomfile) {
 
 isc_result_t
 pk11_get_session(pk11_context_t *ctx, pk11_optype_t optype,
-		 isc_boolean_t need_services, isc_boolean_t rw,
-		 isc_boolean_t logon, const char *pin, CK_SLOT_ID slot)
+		 bool need_services, bool rw,
+		 bool logon, const char *pin, CK_SLOT_ID slot)
 {
 	pk11_token_t *token = NULL;
 	pk11_sessionlist_t *freelist;
@@ -407,9 +412,13 @@ pk11_get_session(pk11_context_t *ctx, pk11_optype_t optype,
 	/* Override the token's PIN */
 	if (logon && pin != NULL && *pin != '\0') {
 		if (strlen(pin) > PINLEN)
-			return ISC_R_RANGE;
-		memset(token->pin, 0, PINLEN + 1);
-		strncpy(token->pin, pin, PINLEN);
+			return (ISC_R_RANGE);
+		/*
+		 * We want to zero out the old pin before
+		 * overwriting with a new one.
+		 */
+		memset(token->pin, 0, sizeof(token->pin));
+		strlcpy(token->pin, pin, sizeof(token->pin));
 	}
 
 	freelist = &token->sessions;
@@ -504,6 +513,7 @@ free_session_list(pk11_sessionlist_t *slist) {
 	LOCK(&sessionlock);
 	while (!ISC_LIST_EMPTY(*slist)) {
 		sp = ISC_LIST_HEAD(*slist);
+		ISC_LIST_UNLINK(*slist, sp, link);
 		UNLOCK(&sessionlock);
 		if (sp->session != CK_INVALID_HANDLE) {
 			rv = pkcs_C_CloseSession(sp->session);
@@ -511,7 +521,6 @@ free_session_list(pk11_sessionlist_t *slist) {
 				ret = DST_R_CRYPTOFAILURE;
 		}
 		LOCK(&sessionlock);
-		ISC_LIST_UNLINK(*slist, sp, link);
 		pk11_mem_put(sp, sizeof(*sp));
 	}
 	UNLOCK(&sessionlock);
@@ -521,7 +530,7 @@ free_session_list(pk11_sessionlist_t *slist) {
 
 static isc_result_t
 setup_session(pk11_session_t *sp, pk11_token_t *token,
-	      isc_boolean_t rw)
+	      bool rw)
 {
 	CK_RV rv;
 	CK_FLAGS flags = CKF_SERIAL_SESSION;
@@ -554,7 +563,7 @@ token_login(pk11_session_t *sp) {
 					      "pkcs_C_Login", rv);
 #endif
 		} else
-			token->logged = ISC_TRUE;
+			token->logged = true;
 	}
 	UNLOCK(&sessionlock);
 	return (ret);
@@ -579,7 +588,7 @@ scan_slots(void) {
 	CK_ULONG slotCount;
 	pk11_token_t *token;
 	unsigned int i;
-	isc_boolean_t bad;
+	bool bad;
 
 	slotCount = 0;
 	PK11_FATALCHECK(pkcs_C_GetSlotList, (CK_FALSE, NULL_PTR, &slotCount));
@@ -587,7 +596,7 @@ scan_slots(void) {
 	/* it's not an error if we didn't find any providers */
 	if (slotCount == 0)
 		return;
-	slotList = pk11_mem_get(sizeof(CK_SLOT_ID_PTR) * slotCount);
+	slotList = pk11_mem_get(sizeof(CK_SLOT_ID) * slotCount);
 	RUNTIME_CHECK(slotList != NULL);
 	PK11_FATALCHECK(pkcs_C_GetSlotList, (CK_FALSE, slotList, &slotCount));
 
@@ -618,12 +627,12 @@ scan_slots(void) {
 			rand_token = token;
 
 	try_rsa:
-		bad = ISC_FALSE;
+		bad = false;
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_RSA_PKCS_KEY_PAIR_GEN,
 					     &mechInfo);
 		if ((rv != CKR_OK) ||
 		    ((mechInfo.flags & CKF_GENERATE_KEY_PAIR) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_RSA_PKCS_KEY_PAIR_GEN);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_MD5_RSA_PKCS,
@@ -632,7 +641,7 @@ scan_slots(void) {
 		    ((mechInfo.flags & CKF_SIGN) == 0) ||
 		    ((mechInfo.flags & CKF_VERIFY) == 0)) {
 #if !defined(PK11_MD5_DISABLE) && !defined(PK11_RSA_PKCS_REPLACE)
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_MD5_RSA_PKCS);
 		}
@@ -642,7 +651,7 @@ scan_slots(void) {
 		    ((mechInfo.flags & CKF_SIGN) == 0) ||
 		    ((mechInfo.flags & CKF_VERIFY) == 0)) {
 #ifndef PK11_RSA_PKCS_REPLACE
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_SHA1_RSA_PKCS);
 		}
@@ -652,7 +661,7 @@ scan_slots(void) {
 		    ((mechInfo.flags & CKF_SIGN) == 0) ||
 		    ((mechInfo.flags & CKF_VERIFY) == 0)) {
 #ifndef PK11_RSA_PKCS_REPLACE
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_SHA256_RSA_PKCS);
 		}
@@ -662,7 +671,7 @@ scan_slots(void) {
 		    ((mechInfo.flags & CKF_SIGN) == 0) ||
 		    ((mechInfo.flags & CKF_VERIFY) == 0)) {
 #ifndef PK11_RSA_PKCS_REPLACE
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_SHA512_RSA_PKCS);
 		}
@@ -671,7 +680,7 @@ scan_slots(void) {
 		    ((mechInfo.flags & CKF_SIGN) == 0) ||
 		    ((mechInfo.flags & CKF_VERIFY) == 0)) {
 #ifdef PK11_RSA_PKCS_REPLACE
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_RSA_PKCS);
 		}
@@ -682,12 +691,12 @@ scan_slots(void) {
 			best_rsa_token = token;
 
 	try_dsa:
-		bad = ISC_FALSE;
+		bad = false;
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_DSA_PARAMETER_GEN,
 					     &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_GENERATE) == 0)) {
 #ifndef PK11_DSA_PARAMETER_GEN_SKIP
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_DSA_PARAMETER_GEN);
 		}
@@ -695,14 +704,14 @@ scan_slots(void) {
 					     &mechInfo);
 		if ((rv != CKR_OK) ||
 		    ((mechInfo.flags & CKF_GENERATE_KEY_PAIR) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_DSA_PARAMETER_GEN);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_DSA_SHA1, &mechInfo);
 		if ((rv != CKR_OK) ||
 		    ((mechInfo.flags & CKF_SIGN) == 0) ||
 		    ((mechInfo.flags & CKF_VERIFY) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_DSA_SHA1);
 		}
 		if (bad)
@@ -714,7 +723,7 @@ scan_slots(void) {
 #endif
 
 	try_dh:
-		bad = ISC_FALSE;
+		bad = false;
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_DH_PKCS_PARAMETER_GEN,
 					     &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_GENERATE) == 0)) {
@@ -725,14 +734,14 @@ scan_slots(void) {
 		if ((rv != CKR_OK) ||
 		    ((mechInfo.flags & CKF_GENERATE_KEY_PAIR) == 0)) {
 #ifndef PK11_DH_PKCS_PARAMETER_GEN_SKIP
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_DH_PKCS_KEY_PAIR_GEN);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_DH_PKCS_DERIVE,
 					     &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_DERIVE) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_DH_PKCS_DERIVE);
 		}
 		if (bad)
@@ -744,78 +753,78 @@ scan_slots(void) {
 #endif
 
 	try_digest:
-		bad = ISC_FALSE;
+		bad = false;
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_MD5, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_DIGEST) == 0)) {
 #ifndef PK11_MD5_DISABLE
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_MD5);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_SHA_1, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_DIGEST) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_SHA_1);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_SHA224, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_DIGEST) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_SHA224);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_SHA256, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_DIGEST) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_SHA256);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_SHA384, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_DIGEST) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_SHA384);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_SHA512, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_DIGEST) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_SHA512);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_MD5_HMAC, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_SIGN) == 0)) {
 #if !defined(PK11_MD5_DISABLE) && !defined(PK11_MD5_HMAC_REPLACE)
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_MD5_HMAC);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_SHA_1_HMAC, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_SIGN) == 0)) {
 #ifndef PK11_SHA_1_HMAC_REPLACE
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_SHA_1_HMAC);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_SHA224_HMAC, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_SIGN) == 0)) {
 #ifndef PK11_SHA224_HMAC_REPLACE
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_SHA224_HMAC);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_SHA256_HMAC, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_SIGN) == 0)) {
 #ifndef PK11_SHA256_HMAC_REPLACE
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_SHA256_HMAC);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_SHA384_HMAC, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_SIGN) == 0)) {
 #ifndef PK11_SHA384_HMAC_REPLACE
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_SHA384_HMAC);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_SHA512_HMAC, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_SIGN) == 0)) {
 #ifndef PK11_SHA512_HMAC_REPLACE
-			bad = ISC_TRUE;
+			bad = true;
 #endif
 			PK11_TRACEM(CKM_SHA512_HMAC);
 		}
@@ -830,14 +839,14 @@ scan_slots(void) {
 					     &mechInfo);
 		if ((rv != CKR_OK) ||
 		    ((mechInfo.flags & CKF_GENERATE_KEY_PAIR) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_EC_KEY_PAIR_GEN);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_ECDSA, &mechInfo);
 		if ((rv != CKR_OK) ||
 		    ((mechInfo.flags & CKF_SIGN) == 0) ||
 		    ((mechInfo.flags & CKF_VERIFY) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_ECDSA);
 		}
 		if (bad)
@@ -847,18 +856,18 @@ scan_slots(void) {
 			best_ec_token = token;
 
 	try_gost:
-		bad = ISC_FALSE;
+		bad = false;
 		/* does GOST require digest too? */
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_GOSTR3411, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_DIGEST) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_GOSTR3411);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_GOSTR3410_KEY_PAIR_GEN,
 					     &mechInfo);
 		if ((rv != CKR_OK) ||
 		    ((mechInfo.flags & CKF_GENERATE_KEY_PAIR) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_GOSTR3410_KEY_PAIR_GEN);
 		}
 		rv = pkcs_C_GetMechanismInfo(slot,
@@ -867,20 +876,41 @@ scan_slots(void) {
 		if ((rv != CKR_OK) ||
 		    ((mechInfo.flags & CKF_SIGN) == 0) ||
 		    ((mechInfo.flags & CKF_VERIFY) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_GOSTR3410_WITH_GOSTR3411);
 		}
 		if (bad)
-			goto try_aes;
+			goto try_eddsa;
 		token->operations |= 1 << OP_GOST;
 		if (best_gost_token == NULL)
 			best_gost_token = token;
 
+	try_eddsa:
+#if defined(CKM_EDDSA_KEY_PAIR_GEN) && defined(CKM_EDDSA) && defined(CKK_EDDSA)
+		bad = false;
+		rv = pkcs_C_GetMechanismInfo(slot, CKM_EDDSA_KEY_PAIR_GEN,
+					     &mechInfo);
+		if ((rv != CKR_OK) ||
+		    ((mechInfo.flags & CKF_GENERATE_KEY_PAIR) == 0)) {
+			bad = true;
+			PK11_TRACEM(CKM_EDDSA_KEY_PAIR_GEN);
+		}
+		rv = pkcs_C_GetMechanismInfo(slot, CKM_EDDSA, &mechInfo);
+		if ((rv != CKR_OK) ||
+		    ((mechInfo.flags & CKF_SIGN) == 0) ||
+		    ((mechInfo.flags & CKF_VERIFY) == 0)) {
+			bad = true;
+			PK11_TRACEM(CKM_EDDSA);
+		}
+		if (bad)
+			goto try_aes;
+
 	try_aes:
-		bad = ISC_FALSE;
+#endif
+		bad = false;
 		rv = pkcs_C_GetMechanismInfo(slot, CKM_AES_ECB, &mechInfo);
 		if ((rv != CKR_OK) || ((mechInfo.flags & CKF_ENCRYPT) == 0)) {
-			bad = ISC_TRUE;
+			bad = true;
 			PK11_TRACEM(CKM_AES_ECB);
 		}
 		if (bad)
@@ -890,8 +920,9 @@ scan_slots(void) {
 			aes_token = token;
 	}
 
-	if (slotList != NULL)
-		pk11_mem_put(slotList, sizeof(CK_SLOT_ID_PTR) * slotCount);
+	if (slotList != NULL) {
+		pk11_mem_put(slotList, sizeof(CK_SLOT_ID) * slotCount);
+	}
 }
 
 CK_SLOT_ID
@@ -1080,7 +1111,7 @@ percent_decode(char *x, size_t *len) {
 	return (x);
 }
 
-static isc_boolean_t
+static bool
 pk11strcmp(const char *x, size_t lenx, const char *y, size_t leny) {
 	char buf[32];
 
@@ -1090,7 +1121,7 @@ pk11strcmp(const char *x, size_t lenx, const char *y, size_t leny) {
 	if (lenx > leny)
 		lenx = leny;
 	memmove(buf, x, lenx);
-	return (ISC_TF(memcmp(buf, y, leny) == 0));
+	return (memcmp(buf, y, leny) == 0);
 }
 
 static CK_ATTRIBUTE *
@@ -1136,7 +1167,7 @@ pk11_parse_uri(pk11_object_t *obj, const char *label,
 	size_t len, l;
 	FILE *stream = NULL;
 	char pin[PINLEN + 1];
-	isc_boolean_t gotpin = ISC_FALSE;
+	bool gotpin = false;
 	isc_result_t ret;
 
 	/* get values to work on */
@@ -1253,7 +1284,7 @@ pk11_parse_uri(pk11_object_t *obj, const char *label,
 			stream = NULL;
 			if (ret != ISC_R_SUCCESS)
 				goto err;
-			gotpin = ISC_TRUE;
+			gotpin = true;
 		} else
 			DST_RET(PK11_R_NOPROVIDER);
 	}
@@ -1277,7 +1308,7 @@ pk11_parse_uri(pk11_object_t *obj, const char *label,
 	obj->slot = token->slotid;
 	if (gotpin) {
 		memmove(token->pin, pin, PINLEN + 1);
-		obj->reqlogon = ISC_TRUE;
+		obj->reqlogon = true;
 	}
 
 	ret = ISC_R_SUCCESS;
@@ -1299,7 +1330,7 @@ pk11_error_fatalcheck(const char *file, int line,
 void
 pk11_dump_tokens(void) {
 	pk11_token_t *token;
-	isc_boolean_t first;
+	bool first;
 
 	printf("DEFAULTS\n");
 	printf("\trand_token=%p\n", rand_token);
@@ -1322,47 +1353,45 @@ pk11_dump_tokens(void) {
 		printf("\tmodel=%.16s\n", token->model);
 		printf("\tserialNumber=%.16s\n", token->serial);
 		printf("\tsupported operations=0x%x (", token->operations);
-		first = ISC_TRUE;
+		first = true;
 		if (token->operations & (1 << OP_RAND)) {
 			if (!first)
 				printf(",");
-			first = ISC_FALSE;
+			first = false;
 			printf("RAND");
 		}
 		if (token->operations & (1 << OP_RSA)) {
-			if (!first)
-				printf(",");
-			first = ISC_FALSE;
+			first = false;
 			printf("RSA");
 		}
 		if (token->operations & (1 << OP_DSA)) {
 			if (!first)
 				printf(",");
-			first = ISC_FALSE;
+			first = false;
 			printf("DSA");
 		}
 		if (token->operations & (1 << OP_DH)) {
 			if (!first)
 				printf(",");
-			first = ISC_FALSE;
+			first = false;
 			printf("DH");
 		}
 		if (token->operations & (1 << OP_DIGEST)) {
 			if (!first)
 				printf(",");
-			first = ISC_FALSE;
+			first = false;
 			printf("DIGEST");
 		}
 		if (token->operations & (1 << OP_EC)) {
 			if (!first)
 				printf(",");
-			first = ISC_FALSE;
+			first = false;
 			printf("EC");
 		}
 		if (token->operations & (1 << OP_AES)) {
 			if (!first)
 				printf(",");
-			first = ISC_FALSE;
+			first = false;
 			printf("AES");
 		}
 		printf(")\n");
