@@ -1,17 +1,12 @@
 /*
- * Copyright (C) 2011-2016  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
 /* ! \file */
@@ -20,6 +15,7 @@
 
 #include <atf-c.h>
 
+#include <inttypes.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -46,8 +42,6 @@ const char *s;
 char str[2 * ISC_SHA512_DIGESTLENGTH + 3];
 unsigned char key[20];
 
-isc_result_t
-tohexstr(unsigned char *d, unsigned int len, char *out);
 /*
  * Precondition: a hexadecimal number in *d, the length of that number in len,
  *   and a pointer to a character array to put the output (*out).
@@ -58,19 +52,17 @@ tohexstr(unsigned char *d, unsigned int len, char *out);
  *
  * Return values: ISC_R_SUCCESS if the operation is sucessful
  */
-
-isc_result_t
-tohexstr(unsigned char *d, unsigned int len, char *out) {
-
-	out[0]='\0';
+static isc_result_t
+tohexstr(unsigned char *d, unsigned int len, char *out, size_t out_size) {
 	char c_ret[] = "AA";
 	unsigned int i;
-	strcat(out, "0x");
+
+	out[0] = '\0';
+	strlcat(out, "0x", out_size);
 	for (i = 0; i < len; i++) {
-		sprintf(c_ret, "%02X", d[i]);
-		strcat(out, c_ret);
+		snprintf(c_ret, sizeof(c_ret), "%02X", d[i]);
+		strlcat(out, c_ret, out_size);
 	}
-	strcat(out, "\0");
 	return (ISC_R_SUCCESS);
 }
 
@@ -209,11 +201,11 @@ ATF_TC_BODY(isc_sha1, tc) {
 		isc_sha1_init(&sha1);
 		for(i = 0; i < testcase->repeats; i++) {
 			isc_sha1_update(&sha1,
-					(const isc_uint8_t *) testcase->input,
+					(const uint8_t *) testcase->input,
 					testcase->input_len);
 		}
 		isc_sha1_final(&sha1, digest);
-		tohexstr(digest, ISC_SHA1_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_SHA1_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -336,7 +328,7 @@ ATF_TC_BODY(isc_sha224, tc) {
 		isc_sha224_init(&sha224);
 		for(i = 0; i < testcase->repeats; i++) {
 			isc_sha224_update(&sha224,
-					  (const isc_uint8_t *) testcase->input,
+					  (const uint8_t *) testcase->input,
 					  testcase->input_len);
 		}
 		isc_sha224_final(digest, &sha224);
@@ -346,7 +338,7 @@ ATF_TC_BODY(isc_sha224, tc) {
 		* functions the call should be
 		* isc_sha224_final(&sha224, digest);
 		 */
-		tohexstr(digest, ISC_SHA224_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_SHA224_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -468,7 +460,7 @@ ATF_TC_BODY(isc_sha256, tc) {
 		isc_sha256_init(&sha256);
 		for(i = 0; i < testcase->repeats; i++) {
 			isc_sha256_update(&sha256,
-					  (const isc_uint8_t *) testcase->input,
+					  (const uint8_t *) testcase->input,
 					  testcase->input_len);
 		}
 		isc_sha256_final(digest, &sha256);
@@ -478,7 +470,7 @@ ATF_TC_BODY(isc_sha256, tc) {
 		* functions the call should be
 		* isc_sha224_final(&sha224, digest);
 		 */
-		tohexstr(digest, ISC_SHA256_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_SHA256_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -614,7 +606,7 @@ ATF_TC_BODY(isc_sha384, tc) {
 		isc_sha384_init(&sha384);
 		for(i = 0; i < testcase->repeats; i++) {
 			isc_sha384_update(&sha384,
-					  (const isc_uint8_t *) testcase->input,
+					  (const uint8_t *) testcase->input,
 					  testcase->input_len);
 		}
 		isc_sha384_final(digest, &sha384);
@@ -624,7 +616,7 @@ ATF_TC_BODY(isc_sha384, tc) {
 		* functions the call should be
 		* isc_sha224_final(&sha224, digest);
 		 */
-		tohexstr(digest, ISC_SHA384_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_SHA384_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -761,7 +753,7 @@ ATF_TC_BODY(isc_sha512, tc) {
 		isc_sha512_init(&sha512);
 		for(i = 0; i < testcase->repeats; i++) {
 			isc_sha512_update(&sha512,
-					  (const isc_uint8_t *) testcase->input,
+					  (const uint8_t *) testcase->input,
 					  testcase->input_len);
 		}
 		isc_sha512_final(digest, &sha512);
@@ -771,7 +763,7 @@ ATF_TC_BODY(isc_sha512, tc) {
 		* functions the call should be
 		* isc_sha224_final(&sha224, digest);
 		 */
-		tohexstr(digest, ISC_SHA512_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_SHA512_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -841,11 +833,11 @@ ATF_TC_BODY(isc_md5, tc) {
 		isc_md5_init(&md5);
 		for(i = 0; i < testcase->repeats; i++) {
 			isc_md5_update(&md5,
-				       (const isc_uint8_t *) testcase->input,
+				       (const uint8_t *) testcase->input,
 				       testcase->input_len);
 		}
 		isc_md5_final(&md5, digest);
-		tohexstr(digest, ISC_MD5_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_MD5_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -972,10 +964,10 @@ ATF_TC_BODY(isc_hmacsha1, tc) {
 		memmove(buffer, test_key->key, test_key->len);
 		isc_hmacsha1_init(&hmacsha1, buffer, test_key->len);
 		isc_hmacsha1_update(&hmacsha1,
-				    (const isc_uint8_t *) testcase->input,
+				    (const uint8_t *) testcase->input,
 				    testcase->input_len);
 		isc_hmacsha1_sign(&hmacsha1, digest, ISC_SHA1_DIGESTLENGTH);
-		tohexstr(digest, ISC_SHA1_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_SHA1_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -1135,10 +1127,10 @@ ATF_TC_BODY(isc_hmacsha224, tc) {
 		memmove(buffer, test_key->key, test_key->len);
 		isc_hmacsha224_init(&hmacsha224, buffer, test_key->len);
 		isc_hmacsha224_update(&hmacsha224,
-				      (const isc_uint8_t *) testcase->input,
+				      (const uint8_t *) testcase->input,
 				      testcase->input_len);
 		isc_hmacsha224_sign(&hmacsha224, digest, ISC_SHA224_DIGESTLENGTH);
-		tohexstr(digest, ISC_SHA224_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_SHA224_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -1298,10 +1290,10 @@ ATF_TC_BODY(isc_hmacsha256, tc) {
 		memmove(buffer, test_key->key, test_key->len);
 		isc_hmacsha256_init(&hmacsha256, buffer, test_key->len);
 		isc_hmacsha256_update(&hmacsha256,
-				      (const isc_uint8_t *) testcase->input,
+				      (const uint8_t *) testcase->input,
 				      testcase->input_len);
 		isc_hmacsha256_sign(&hmacsha256, digest, ISC_SHA256_DIGESTLENGTH);
-		tohexstr(digest, ISC_SHA256_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_SHA256_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -1467,10 +1459,10 @@ ATF_TC_BODY(isc_hmacsha384, tc) {
 		memmove(buffer, test_key->key, test_key->len);
 		isc_hmacsha384_init(&hmacsha384, buffer, test_key->len);
 		isc_hmacsha384_update(&hmacsha384,
-				      (const isc_uint8_t *) testcase->input,
+				      (const uint8_t *) testcase->input,
 				      testcase->input_len);
 		isc_hmacsha384_sign(&hmacsha384, digest, ISC_SHA384_DIGESTLENGTH);
-		tohexstr(digest, ISC_SHA384_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_SHA384_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -1636,10 +1628,10 @@ ATF_TC_BODY(isc_hmacsha512, tc) {
 		memmove(buffer, test_key->key, test_key->len);
 		isc_hmacsha512_init(&hmacsha512, buffer, test_key->len);
 		isc_hmacsha512_update(&hmacsha512,
-				      (const isc_uint8_t *) testcase->input,
+				      (const uint8_t *) testcase->input,
 				      testcase->input_len);
 		isc_hmacsha512_sign(&hmacsha512, digest, ISC_SHA512_DIGESTLENGTH);
-		tohexstr(digest, ISC_SHA512_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_SHA512_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -1780,10 +1772,10 @@ ATF_TC_BODY(isc_hmacmd5, tc) {
 		memmove(buffer, test_key->key, test_key->len);
 		isc_hmacmd5_init(&hmacmd5, buffer, test_key->len);
 		isc_hmacmd5_update(&hmacmd5,
-				   (const isc_uint8_t *) testcase->input,
+				   (const uint8_t *) testcase->input,
 				   testcase->input_len);
 		isc_hmacmd5_sign(&hmacmd5, digest);
-		tohexstr(digest, ISC_MD5_DIGESTLENGTH, str);
+		tohexstr(digest, ISC_MD5_DIGESTLENGTH, str, sizeof(str));
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -1798,7 +1790,7 @@ ATF_TC_HEAD(isc_crc64, tc) {
 	atf_tc_set_md_var(tc, "descr", "64-bit cyclic redundancy check");
 }
 ATF_TC_BODY(isc_crc64, tc) {
-	isc_uint64_t crc;
+	uint64_t crc;
 	int i;
 
 	UNUSED(tc);
@@ -1810,29 +1802,29 @@ ATF_TC_BODY(isc_crc64, tc) {
 		},
 		{
 			TEST_INPUT("a"),
-			"0x9AA9C0AC27F473CE", 1
+			"0xCE73F427ACC0A99A", 1
 		},
 		{
 			TEST_INPUT("abc"),
-			"0x0297F4F93A818B04", 1
+			"0x048B813AF9F49702", 1
 		},
 		{
 			TEST_INPUT("message digest"),
-			"0xF47B357AEAF97352", 1
+			"0x5273F9EA7A357BF4", 1
 		},
 		{
 			TEST_INPUT("abcdefghijklmnopqrstuvwxyz"),
-			"0xA1AA8B21F979F059", 1
+			"0x59F079F9218BAAA1", 1
 		},
 		{
 			TEST_INPUT("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklm"
 				   "nopqrstuvwxyz0123456789"),
-			"0xFBB6781EF7A86DA3", 1
+			"0xA36DA8F71E78B6FB", 1
 		},
 		{
 			TEST_INPUT("123456789012345678901234567890123456789"
 				   "01234567890123456789012345678901234567890"),
-			"0x4A87E7C873EBE581", 1
+			"0x81E5EB73C8E7874A", 1
 		},
 		{ NULL, 0, NULL, 1 }
 	};
@@ -1843,11 +1835,12 @@ ATF_TC_BODY(isc_crc64, tc) {
 		isc_crc64_init(&crc);
 		for(i = 0; i < testcase->repeats; i++) {
 			isc_crc64_update(&crc,
-				       (const isc_uint8_t *) testcase->input,
+				       (const uint8_t *) testcase->input,
 				       testcase->input_len);
 		}
 		isc_crc64_final(&crc);
-		tohexstr((unsigned char *) &crc, sizeof(crc), str);
+		snprintf(str, sizeof(str),
+			 "0x%016" PRIX64, crc);
 		ATF_CHECK_STREQ(str, testcase->result);
 
 		testcase++;
@@ -1866,42 +1859,41 @@ ATF_TC_BODY(isc_hash_function, tc) {
 
 	/* Incremental hashing */
 
-	h1 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
-	h1 = isc_hash_function("This ", 5, ISC_TRUE, &h1);
-	h1 = isc_hash_function("is ", 3, ISC_TRUE, &h1);
-	h1 = isc_hash_function("a long test", 12, ISC_TRUE, &h1);
+	h1 = isc_hash_function(NULL, 0, true, NULL);
+	h1 = isc_hash_function("This ", 5, true, &h1);
+	h1 = isc_hash_function("is ", 3, true, &h1);
+	h1 = isc_hash_function("a long test", 12, true, &h1);
 
 	h2 = isc_hash_function("This is a long test", 20,
-			       ISC_TRUE, NULL);
+			       true, NULL);
 
 	ATF_CHECK_EQ(h1, h2);
 
 	/* Immutability of hash function */
-	h1 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
-	h2 = isc_hash_function(NULL, 0, ISC_TRUE, NULL);
+	h1 = isc_hash_function(NULL, 0, true, NULL);
+	h2 = isc_hash_function(NULL, 0, true, NULL);
 
 	ATF_CHECK_EQ(h1, h2);
 
 	/* Hash function characteristics */
-	h1 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
-	h2 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
+	h1 = isc_hash_function("Hello world", 12, true, NULL);
+	h2 = isc_hash_function("Hello world", 12, true, NULL);
 
 	ATF_CHECK_EQ(h1, h2);
 
 	/* Case */
-	h1 = isc_hash_function("Hello world", 12, ISC_FALSE, NULL);
-	h2 = isc_hash_function("heLLo WorLd", 12, ISC_FALSE, NULL);
+	h1 = isc_hash_function("Hello world", 12, false, NULL);
+	h2 = isc_hash_function("heLLo WorLd", 12, false, NULL);
 
 	ATF_CHECK_EQ(h1, h2);
 
 	/* Unequal */
-	h1 = isc_hash_function("Hello world", 12, ISC_TRUE, NULL);
-	h2 = isc_hash_function("heLLo WorLd", 12, ISC_TRUE, NULL);
+	h1 = isc_hash_function("Hello world", 12, true, NULL);
+	h2 = isc_hash_function("heLLo WorLd", 12, true, NULL);
 
 	ATF_CHECK(h1 != h2);
 }
 
-
 ATF_TC(isc_hash_function_reverse);
 ATF_TC_HEAD(isc_hash_function_reverse, tc) {
 	atf_tc_set_md_var(tc, "descr", "Reverse hash function test");
@@ -1914,41 +1906,100 @@ ATF_TC_BODY(isc_hash_function_reverse, tc) {
 
 	/* Incremental hashing */
 
-	h1 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
-	h1 = isc_hash_function_reverse("\000", 1, ISC_TRUE, &h1);
-	h1 = isc_hash_function_reverse("\003org", 4, ISC_TRUE, &h1);
-	h1 = isc_hash_function_reverse("\007example", 8, ISC_TRUE, &h1);
+	h1 = isc_hash_function_reverse(NULL, 0, true, NULL);
+	h1 = isc_hash_function_reverse("\000", 1, true, &h1);
+	h1 = isc_hash_function_reverse("\003org", 4, true, &h1);
+	h1 = isc_hash_function_reverse("\007example", 8, true, &h1);
 
 	h2 = isc_hash_function_reverse("\007example\003org\000", 13,
-				       ISC_TRUE, NULL);
+				       true, NULL);
 
 	ATF_CHECK_EQ(h1, h2);
 
 	/* Immutability of hash function */
-	h1 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
-	h2 = isc_hash_function_reverse(NULL, 0, ISC_TRUE, NULL);
+	h1 = isc_hash_function_reverse(NULL, 0, true, NULL);
+	h2 = isc_hash_function_reverse(NULL, 0, true, NULL);
 
 	ATF_CHECK_EQ(h1, h2);
 
 	/* Hash function characteristics */
-	h1 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
-	h2 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
+	h1 = isc_hash_function_reverse("Hello world", 12, true, NULL);
+	h2 = isc_hash_function_reverse("Hello world", 12, true, NULL);
 
 	ATF_CHECK_EQ(h1, h2);
 
 	/* Case */
-	h1 = isc_hash_function_reverse("Hello world", 12, ISC_FALSE, NULL);
-	h2 = isc_hash_function_reverse("heLLo WorLd", 12, ISC_FALSE, NULL);
+	h1 = isc_hash_function_reverse("Hello world", 12, false, NULL);
+	h2 = isc_hash_function_reverse("heLLo WorLd", 12, false, NULL);
 
 	ATF_CHECK_EQ(h1, h2);
 
 	/* Unequal */
-	h1 = isc_hash_function_reverse("Hello world", 12, ISC_TRUE, NULL);
-	h2 = isc_hash_function_reverse("heLLo WorLd", 12, ISC_TRUE, NULL);
+	h1 = isc_hash_function_reverse("Hello world", 12, true, NULL);
+	h2 = isc_hash_function_reverse("heLLo WorLd", 12, true, NULL);
 
 	ATF_CHECK(h1 != h2);
 }
 
+ATF_TC(isc_hash_initializer);
+ATF_TC_HEAD(isc_hash_initializer, tc) {
+	atf_tc_set_md_var(tc, "descr", "Hash function initializer test");
+}
+ATF_TC_BODY(isc_hash_initializer, tc) {
+	unsigned int h1;
+	unsigned int h2;
+
+	UNUSED(tc);
+
+	h1 = isc_hash_function("Hello world", 12, true, NULL);
+	h2 = isc_hash_function("Hello world", 12, true, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+
+	isc_hash_set_initializer(isc_hash_get_initializer());
+
+	/* Hash value must not change */
+	h2 = isc_hash_function("Hello world", 12, true, NULL);
+
+	ATF_CHECK_EQ(h1, h2);
+}
+
+#ifndef PK11_MD5_DISABLE
+ATF_TC(md5_check);
+ATF_TC_HEAD(md5_check, tc) {
+	atf_tc_set_md_var(tc, "descr", "Startup MD5 check test");
+}
+ATF_TC_BODY(md5_check, tc) {
+	UNUSED(tc);
+
+	ATF_REQUIRE(isc_md5_check(false));
+	ATF_CHECK(!isc_md5_check(true));
+
+	ATF_REQUIRE(isc_hmacmd5_check(0));
+	ATF_CHECK(!isc_hmacmd5_check(1));
+	ATF_CHECK(!isc_hmacmd5_check(2));
+	ATF_CHECK(!isc_hmacmd5_check(3));
+	ATF_CHECK(!isc_hmacmd5_check(4));
+}
+#endif
+
+ATF_TC(sha1_check);
+ATF_TC_HEAD(sha1_check, tc) {
+	atf_tc_set_md_var(tc, "descr", "Startup SHA-1 check test");
+}
+ATF_TC_BODY(sha1_check, tc) {
+	UNUSED(tc);
+
+	ATF_REQUIRE(isc_sha1_check(false));
+	ATF_CHECK(!isc_sha1_check(true));
+
+	ATF_REQUIRE(isc_hmacsha1_check(0));
+	ATF_CHECK(!isc_hmacsha1_check(1));
+	ATF_CHECK(!isc_hmacsha1_check(2));
+	ATF_CHECK(!isc_hmacsha1_check(3));
+	ATF_CHECK(!isc_hmacsha1_check(4));
+}
+
 /*
  * Main
  */
@@ -1957,8 +2008,14 @@ ATF_TP_ADD_TCS(tp) {
 	 * Tests of hash functions, including isc_hash and the
 	 * various cryptographic hashes.
 	 */
+#ifndef PK11_MD5_DISABLE
+	ATF_TP_ADD_TC(tp, md5_check);
+#endif
+	ATF_TP_ADD_TC(tp, sha1_check);
+
 	ATF_TP_ADD_TC(tp, isc_hash_function);
 	ATF_TP_ADD_TC(tp, isc_hash_function_reverse);
+	ATF_TP_ADD_TC(tp, isc_hash_initializer);
 #ifndef PK11_MD5_DISABLE
 	ATF_TP_ADD_TC(tp, isc_hmacmd5);
 #endif
